<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\InformUsaOffice;
use Modules\Sd\Entities\InformUsaOfficeAttachment;
use Modules\Sd\Entities\Shipment;
use Modules\Sd\Entities\ShippingAgent;

class InformUsaOfficeController extends Controller
{

    public function getAllInfousaOffiData()
    {
        try {
            $query = 'SELECT * FROM sd_inform_usa_office ORDER BY `created_at` DESC';
            $furmugations_data = DB::select($query);
            $result = array();

            foreach ($furmugations_data as $data) {
                array_push($result, [
                    "customer_order_id" => $data->customer_order_id,
                    "factory_po" => $data->factory_po,
                    "shipment_id" => $data->shipment_id,
                    "inform_usa_office_id" => $data->inform_usa_office_id,
                ]);
            }
            $responseBody = $this->responseBody(true, "informUsaOfficeController", "getAllInfousaOffiData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "getAllInfousaOffiData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function saveData(Request $request)
    {
        try {
            $inform_data = new InformUsaOffice();
            $inform_data->customer_order_id = $request->get('customer_order_id');
            $inform_data->factory_po = $request->get('factory_po');
            $inform_data->shipment_id = $request->get('shipment_id');
            $inform_data->note = $request->get('note');

            if ($inform_data->save()) {
                // $this->sendEmail();
                $attachments = InformUsaOfficeAttachment::where([['inform_usa_office_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->inform_usa_office_id = $inform_data->inform_usa_office_id;
                    $attachment->update();
                }
                $attachments = InformUsaOfficeAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->inform_usa_office_id = $inform_data->inform_usa_office_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "informUsaOfficeController", "saveData", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "saveData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function update(Request $request, $id)
    {
        try {
            $infrom_data = InformUsaOffice::find($id);
            $infrom_data->customer_order_id = $request->get('customer_order_id');
            $infrom_data->factory_po = $request->get('factory_po');
            $infrom_data->shipment_id = $request->get('shipment_id');
            $infrom_data->note = $request->get('note');
            if ($infrom_data->update()) {
                $attachments = InformUsaOfficeAttachment::where([['inform_usa_office_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->inform_usa_office_id = $infrom_data->inform_usa_office_id;
                    $attachment->update();
                }
                $attachments = InformUsaOfficeAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->inform_usa_office_id = $infrom_data->inform_usa_office_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "informUsaOfficeController", "update", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "update", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function delete(Request $request, $id)
    {
        try {
            $inform_data = InformUsaOffice::where('inform_usa_office_id', '=', $id);
            $rec_data_attachments = InformUsaOfficeAttachment::where('inform_usa_office_id', '=', $id)->get();

            if ($inform_data->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "informUsaOfficeController", "delete", "success");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "delete", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getInfoUsaOffiData($id)
    {
        try {
            $recordHead = InformUsaOffice::find($id);

            $responseBody = $this->responseBody(true, "informUsaOfficeController", "getInfoUsaOffiData", $recordHead);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShipmentData()
    {
        try {
            $shipmet = Shipment::all();

            $responseBody = $this->responseBody(true, "informUsaOfficeController", "getShipmentData", $shipmet);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "getShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShippingAgentData()
    {
        try {
            $shipping_agent = ShippingAgent::all();

            $responseBody = $this->responseBody(true, "informUsaOfficeController", "getShippingAgentData", $shipping_agent);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "getShippingAgentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            sd_shipment.customer_order_id,
            sd_shipment.shipment_id,
            pp_customer_orders.factory_po_num
        FROM
            sd_shipment
            LEFT JOIN pp_customer_orders ON sd_shipment.customer_order_id = pp_customer_orders.order_id
        WHERE
            sd_shipment.shipment_id = ' . $id;
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_id" => $order->customer_order_id,
                    "factory_po_num" => $order->factory_po_num,
                    "shipment_id" => $order->shipment_id
                ]);
            }


            $responseBody = $this->responseBody(true, "informUsaOfficeController", "getCustomerOrderData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = url('/') .'/infrom_usa_office/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('infrom_usa_office'), $filename);

        $inform_attch_data = new InformUsaOfficeAttachment();
        $inform_attch_data->inform_usa_office_id = 0;
        $inform_attch_data->description = $description;
        $inform_attch_data->file_path = $filename;
        $inform_attch_data->token = $token;
        return $inform_attch_data->save();
    }

    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sd_inform_usa_office_attachments WHERE  inform_usa_office_id = "' . $id . '" OR inform_usa_office_id = "0"');
            $responseBody = $this->responseBody(true, "informUsaOfficeController", "allScheduleAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "allScheduleAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = InformUsaOfficeAttachment::where('inform_usa_office_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "informUsaOfficeController", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "informUsaOfficeController", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
