<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\PreshipmentDocumnet;
use Modules\Sd\Entities\PreshipmentDocumnetAttachment;
use Modules\Sd\Entities\Shipment;
use Modules\Sd\Entities\ShippingAgent;

class PreshipmentDocumnetController extends Controller
{
    public function getAllPreshipmentDocData()
    {
        try {
            $query = 'SELECT * FROM sd_preshipment_document ORDER BY `created_at` DESC';
            $clearance_doc_usa_datas = DB::select($query);
            $result = array();

            foreach ($clearance_doc_usa_datas as $clearance_doc_usa_data) {
                array_push($result, [
                    "customer_order_id" => $clearance_doc_usa_data->customer_order_id,
                    "factory_po" => $clearance_doc_usa_data->factory_po,
                    "shipment_id" => $clearance_doc_usa_data->shipment_id,
                    "preshipment_document_id" => $clearance_doc_usa_data->preshipment_document_id,
                ]);
            }
            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "getAllPreshipmentDocData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "getAllPreshipmentDocData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function savePreshipmentDoc(Request $request)
    {
        try {
            $preshipment_doc = new PreshipmentDocumnet();
            $preshipment_doc->customer_order_id = $request->get('customer_order_id');
            $preshipment_doc->factory_po = $request->get('factory_po');
            $preshipment_doc->shipment_id = $request->get('shipment_id');
            $preshipment_doc->note = $request->get('note');

            if ($preshipment_doc->save()) {
                // $this->sendEmail();
                $attachments = PreshipmentDocumnetAttachment::where([['preshipment_document_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->preshipment_document_id = $preshipment_doc->preshipment_document_id;
                    $attachment->update();
                }
                $attachments = PreshipmentDocumnetAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->preshipment_document_id = $preshipment_doc->preshipment_document_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "savePreshipmentDoc", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "savePreshipmentDoc", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function update(Request $request, $id)
    {
        try {
            $preshipment_doc = PreshipmentDocumnet::find($id);
            $preshipment_doc->customer_order_id = $request->get('customer_order_id');
            $preshipment_doc->factory_po = $request->get('factory_po');
            $preshipment_doc->shipment_id = $request->get('shipment_id');
            $preshipment_doc->note = $request->get('note');
            if ($preshipment_doc->update()) {
                $attachments = PreshipmentDocumnetAttachment::where([['preshipment_document_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->preshipment_document_id = $preshipment_doc->preshipment_document_id;
                    $attachment->update();
                }
                $attachments = PreshipmentDocumnetAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->preshipment_document_id = $preshipment_doc->preshipment_document_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "update", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "update", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function delete(Request $request, $id)
    {
        try {
            $preshipment_doc = PreshipmentDocumnet::where('preshipment_document_id', '=', $id);
            $rec_data_attachments = PreshipmentDocumnetAttachment::where('preshipment_document_id', '=', $id)->get();

            if ($preshipment_doc->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "delete", "success");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "delete", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getPreshipmentDocData($id)
    {
        try {
            $recordHead = PreshipmentDocumnet::find($id);

            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "getCdDocumentData", $recordHead);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShipmentData()
    {
        try {
            $shipmet = Shipment::all();

            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "getShipmentData", $shipmet);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "getShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShippingAgentData()
    {
        try {
            $shipping_agent = ShippingAgent::all();

            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "getShippingAgentData", $shipping_agent);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "getShippingAgentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            sd_shipment.customer_order_id,
            sd_shipment.shipment_id,
            pp_customer_orders.factory_po_num
        FROM
            sd_shipment
            LEFT JOIN pp_customer_orders ON sd_shipment.customer_order_id = pp_customer_orders.order_id
        WHERE
            sd_shipment.shipment_id = ' . $id;
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_id" => $order->customer_order_id,
                    "factory_po_num" => $order->factory_po_num,
                    "shipment_id" => $order->shipment_id,
                ]);
            }


            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "getCustomerOrderData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = url('/').'/preshipment_document/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('preshipment_document'), $filename);

        $preshipment_doc = new PreshipmentDocumnetAttachment();
        $preshipment_doc->preshipment_document_id = 0;
        $preshipment_doc->description = $description;
        $preshipment_doc->file_path = $filename;
        $preshipment_doc->token = $token;
        return $preshipment_doc->save();
    }

    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sd_preshipment_document_attachment WHERE  preshipment_document_id = "' . $id . '" OR preshipment_document_id = "0"');
            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "allCdUsaAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "allCdUsaAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = PreshipmentDocumnetAttachment::where('preshipment_document_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "PreshipmentDocumentController", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "PreshipmentDocumentController", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
