<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\Shipment;
use Modules\Sd\Entities\ShippingAgent;
use Modules\Sd\Entities\SpecialDocument;
use Modules\Sd\Entities\SpecialDocumentAttachment;

class SpecialDocumentController extends Controller
{
    public function getAllSdData()
    {
        try {
            $query = 'SELECT * FROM sd_special_document ORDER BY `created_at` DESC';
            $sp_docs = DB::select($query);
            $result = array();

            foreach ($sp_docs as $sp_doc) {
                array_push($result, [
                    "customer_order_id" => $sp_doc->customer_order_id,
                    "factory_po" => $sp_doc->factory_po,
                    "shipment_id" => $sp_doc->shipment_id,
                    "special_document_id" => $sp_doc->special_document_id,
                ]);
            }
            $responseBody = $this->responseBody(true, "SpecialDocumentController", "getAllSdData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "getAllSdData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function saveData(Request $request)
    {
        try {
            $sp_doc = new SpecialDocument();
            $sp_doc->customer_order_id = $request->get('customer_order_id');
            $sp_doc->factory_po = $request->get('factory_po');
            $sp_doc->shipment_id = $request->get('shipment_id');
            $sp_doc->note = $request->get('note');

            if ($sp_doc->save()) {
                // $this->sendEmail();
                $attachments = SpecialDocumentAttachment::where([['special_document_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->special_document_id = $sp_doc->special_document_id;
                    $attachment->update();
                }
                $attachments = SpecialDocumentAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->special_document_id = $sp_doc->special_document_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "SpecialDocumentController", "saveData", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "saveData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function update(Request $request, $id)
    {
        try {
            $sp_doc = SpecialDocument::find($id);
            $sp_doc->customer_order_id = $request->get('customer_order_id');
            $sp_doc->factory_po = $request->get('factory_po');
            $sp_doc->shipment_id = $request->get('shipment_id');
            $sp_doc->note = $request->get('note');
            if ($sp_doc->update()) {
                $attachments = SpecialDocumentAttachment::where([['special_document_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->special_document_id = $sp_doc->special_document_id;
                    $attachment->update();
                }
                $attachments = SpecialDocumentAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->special_document_id = $sp_doc->special_document_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "SpecialDocumentController", "update", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "update", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function delete(Request $request, $id)
    {
        try {
            $sp_doc = SpecialDocument::where('special_document_id', '=', $id);
            $rec_data_attachments = SpecialDocumentAttachment::where('special_document_id', '=', $id)->get();

            if ($sp_doc->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "specialDocumentController", "delete", "success");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "specialDocumentController", "delete", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getSpDocData($id)
    {
        try {
            $recordHead = SpecialDocument::find($id);

            $responseBody = $this->responseBody(true, "SpecialDocumentController", "getSpDocData", $recordHead);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShipmentData()
    {
        try {
            $shipmet = Shipment::all();

            $responseBody = $this->responseBody(true, "SpecialDocumentController", "getShipmentData", $shipmet);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "getShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShippingAgentData()
    {
        try {
            $shipping_agent = ShippingAgent::all();

            $responseBody = $this->responseBody(true, "SpecialDocumentController", "getShippingAgentData", $shipping_agent);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "getShippingAgentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            sd_shipment.customer_order_id,
            pp_customer_orders.factory_po_num
        FROM
            sd_shipment
            LEFT JOIN pp_customer_orders ON sd_shipment.customer_order_id = pp_customer_orders.order_id
        WHERE
            sd_shipment.shipment_id = ' . $id;
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_id" => $order->customer_order_id,
                    "factory_po_num" => $order->factory_po_num
                ]);
            }


            $responseBody = $this->responseBody(true, "SpecialDocumentController", "getCustomerOrderData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = url('/') .'/special_document/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('special_document'), $filename);

        $sp_doc = new SpecialDocumentAttachment();
        $sp_doc->special_document_id = 0;
        $sp_doc->description = $description;
        $sp_doc->file_path = $filename;
        $sp_doc->token = $token;
        return $sp_doc->save();
    }

    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sd_special_document_attachment WHERE  special_document_id = "' . $id . '" OR special_document_id = "0"');
            $responseBody = $this->responseBody(true, "SpecialDocumentController", "allScheduleAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "allScheduleAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = SpecialDocumentAttachment::where('special_document_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "SpecialDocumentController", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SpecialDocumentController", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
